@@ -1,7 +1,7 @@
 
 ###################################################################################
 #
-#   Embperl - Copyright (c) 1997-2005 Gerald Richter / ecos gmbh   www.ecos.de
+#   Embperl - Copyright (c) 1997-2010 Gerald Richter / ecos gmbh   www.ecos.de
 #
 #   You may distribute under the terms of either the GNU General Public
 #   License or the Artistic License, as specified in the Perl README file.
@@ -22,6 +22,30 @@ use base 'Embperl::Form::ControlMultValue' ;
 
 use Embperl::Inline ;
 
+# ---------------------------------------------------------------------------
+#
+#   get_select_values - return values of control
+#
+
+sub get_select_values
+    {
+    my ($self, $req) = @_ ;
+
+    my ($values, $options) = $self -> get_values ($req) ;
+    my $addtop   = $self -> {addtop} || [] ;
+    my $addbottom= $self -> {addbottom} || [] ;
+    my $addtop_options    = [] ;
+    my $addbottom_options = [] ;
+    if (!$self -> {showoptions})
+	{
+    	$addtop_options    = $self -> form -> convert_options ($self, $addtop) if ($self -> {addtop}) ;
+    	$addbottom_options = $self -> form -> convert_options ($self, $addbottom) if ($self -> {addbottom}) ;
+	}
+
+    return ($values, $options, $addtop, $addtop_options, $addbottom, $addbottom_options) ;
+    }
+
+
 # ---------------------------------------------------------------------------
 #
 #   show_control_readonly - output readonly control
@@ -32,7 +56,58 @@ sub show_control_readonly
     my ($self, $req) = @_ ;
 
     my $name     = $self -> {name} ;
-    $self -> show_control ($req, "^\Q$fdat{$name}\\E\$") ;
+
+    my ($values, $options, $addtop, $addtop_options, $addbottom, $addbottom_options) = $self -> get_select_values ($req) ;
+
+    my $val = $fdat{$name} ;
+    my $opt ;
+    my $i = 0 ;
+    foreach (@$addtop)
+	{
+	if ($_ eq $val)
+	    {
+	    $opt = $addtop_options -> [$i] || $val ;
+	    last ;
+	    }
+	$i++ ;
+	}
+
+    if (!$opt)
+	{
+	$i = 0 ;
+        foreach (@$values)
+	    {
+	    if ($_ eq $val)
+	        {
+	        $opt = $options -> [$i] || $val ;
+	        last ;
+	        }
+	    $i++ ;
+	    }
+        }
+
+    if (!$opt)
+	{
+        $i = 0 ;
+        foreach (@$addbottom)
+	    {
+	    if ($_ eq $val)
+	        {
+	        $opt = $addbottom_options -> [$i] || $val ;
+	        last ;
+	        }
+	    $i++ ;
+	    }
+        }
+
+    if (!$opt)
+	{
+	$opt = $self -> form -> convert_text ($self, 'err:select_not_found') ;
+	}
+
+    $self -> {value} = $opt ;
+    $self -> show_hidden ($req) ;
+    $self -> SUPER::show_control_readonly ($req) ;
     }
 
 
@@ -41,6 +116,15 @@ sub show_control_readonly
 
 __EMBPERL__
 
+[# ---------------------------------------------------------------------------
+#
+#   show_hidden - out hidden field
+#]
+
+[$ sub show_hidden ($self, $req) $]
+<input type="hidden" name="[+ $self -> {name} +]">
+[$endsub$]
+
 [# ---------------------------------------------------------------------------
 #
 #   show_control - output the control
@@ -48,34 +132,39 @@ __EMBPERL__
 
 [$ sub show_control ($self, $req, $filter)
 
-    my ($values, $options) = $self -> get_values ($req) ;
     my $name     = $self -> {name} ;
     $filter      ||= $self -> {filter} ;
-    my $addtop   = $self -> {addtop} || [] ;
-    my $addbottom= $self -> {addbottom} || [] ;
     my $nsprefix = $self -> form -> {jsnamespace} ;
     my $val ;
     my $i = 0 ;
+    my ($values, $options, $addtop, $addtop_options, $addbottom, $addbottom_options) = $self -> get_select_values ($req) ;
+
 $]
-<select  class="cBase cControl" name="[+ $name +]" id="[+ $name +]"
+<select [+ $self->{multiple}?'multiple':''+] class="cBase cControl cControlWidthSelect" name="[+ $name +]" id="[+ $name +]" 
+         
 [$if ($self -> {sublines} || $self -> {subobjects}) $] OnChange="[+ $nsprefix +]show_selected(document, this)" [$endif$]
 [$if ($self -> {rows}) $] size="[+ $self->{rows} +]" [$endif$]
->
+[+ do { local $escmode = 0 ; $self -> {eventattrs} } +]>
+[* $i = 0 ; *]
 [$ foreach $val (@$addtop) $]
     [$if !defined ($filter) || ($val->[0] =~ /$filter/i) $]
-    <option value="[+ $val->[0] +]">[+ $val ->[1] || $val -> [0] +]</option>
+    <option value="[+ $val->[0] +]">[+ $addtop_options -> [$i] || $val ->[1] || $val -> [0] +]</option>
     [$endif$]
+    [* $i++ ; *]
 [$endforeach$]
+[* $i = 0 ; *]
 [$ foreach $val (@$values) $]
     [$if !defined ($filter) || ($val =~ /$filter/i) $]
     <option value="[+ $val +]">[+ $options ->[$i] || $val +]</option>
     [$endif$]
     [* $i++ ; *]
 [$endforeach$]
+[* $i = 0 ; *]
 [$ foreach $val (@$addbottom) $]
     [$if !defined ($filter) || ($val->[0] =~ /$filter/i) $]
-    <option value="[+ $val->[0] +]">[+ $val ->[1] || $val -> [0] +]</option>
+    <option value="[+ $val->[0] +]">[+ $addbottom_options -> [$i] || $val ->[1] || $val -> [0] +]</option>
     [$endif$]
+    [* $i++ ; *]
 [$endforeach$]
 </select>
 
@@ -159,6 +248,9 @@ value (first entry)is displayed. Example:
 If given, only items where the value matches the regex given in
 C<filter> are displayed.
 
+=head3 multiple
+
+If set to true, allows multiple selections.
 
 =head1 Author
 
